@@ -15,24 +15,6 @@ def create_side_effect(returns):
     return side_effect
 
 
-class MockResponse(object):
-    """
-    Mock urllib2.urlopen response.
-    http://stackoverflow.com/a/2276727
-    """
-    def __init__(self, resp_data, code=200, msg='OK'):
-        self.resp_data = resp_data
-        self.code = code
-        self.msg = msg
-        self.headers = {'content-type': 'text/xml; charset=utf-8'}
-
-    def read(self):
-        return self.resp_data
-
-    def getcode(self):
-        return self.code
-
-
 class GCMTest(unittest.TestCase):
 
     def setUp(self):
@@ -71,19 +53,30 @@ class GCMTest(unittest.TestCase):
         }
         time.sleep = MagicMock()
 
+    def test_gcm_proxy(self):
+        self.gcm = GCM('123api', proxy='http://domain.com:8888')
+        self.assertEqual(self.gcm.proxy, {
+            'https': 'http://domain.com:8888'
+        })
+
+        self.gcm = GCM('123api', proxy={
+            'http': 'http://domain.com:8888',
+            'https': 'https://domain.com:8888'
+        })
+        self.assertEqual(self.gcm.proxy, {
+            'http': 'http://domain.com:8888',
+            'https': 'https://domain.com:8888'
+        })
+
     def test_construct_payload(self):
         res = self.gcm.construct_payload(
             registration_ids=['1', '2'], data=self.data, collapse_key='foo',
-            delay_while_idle=True, time_to_live=3600, is_json=True
+            delay_while_idle=True, time_to_live=3600, is_json=True, dry_run = True
         )
         payload = json.loads(res)
-        for arg in ['registration_ids', 'data', 'collapse_key', 'delay_while_idle', 'time_to_live']:
+        for arg in ['registration_ids', 'data', 'collapse_key', 'delay_while_idle', 'time_to_live', 'dry_run']:
             self.assertIn(arg, payload)
 
-    def test_require_collapse_key(self):
-        with self.assertRaises(GCMNoCollapseKeyException):
-            self.gcm.construct_payload(registration_ids='1234', data=self.data, time_to_live=3600)
-
     def test_json_payload(self):
         reg_ids = ['12', '145', '56']
         json_payload = self.gcm.construct_payload(registration_ids=reg_ids, data=self.data)
@@ -94,8 +87,9 @@ class GCMTest(unittest.TestCase):
         self.assertEqual(payload['registration_ids'], reg_ids)
 
     def test_plaintext_payload(self):
-        result = self.gcm.construct_payload(registration_ids='1234', data=self.data, is_json=False)
-
+        result = self.gcm.construct_payload(
+            registration_ids='1234', data=self.data, is_json=False
+        )
         self.assertIn('registration_id', result)
         self.assertIn('data.param1', result)
         self.assertIn('data.param2', result)
@@ -184,43 +178,67 @@ class GCMTest(unittest.TestCase):
         res = self.gcm.handle_plaintext_response(response)
         self.assertEqual(res, '3456')
 
-    @patch('urllib2.urlopen')
-    def test_make_request_plaintext(self, urlopen_mock):
+    @patch('requests.post')
+    def test_make_request_header(self, mock_request):
         """ Test plaintext make_request. """
 
-        # Set mock value for urlopen return value
-        urlopen_mock.return_value = MockResponse('blah')
-
+        mock_request.return_value.status_code = 200
+        mock_request.return_value.content = "OK"
         # Perform request
-        response = self.gcm.make_request({'message': 'test'}, is_json=False)
-
-        # Get request (first positional argument to urlopen)
-        # Ref: http://www.voidspace.org.uk/python/mock/mock.html#mock.Mock.call_args
-        request = urlopen_mock.call_args[0][0]
-
-        # Test encoded data
-        encoded_data = request.get_data()
-        self.assertEquals(
-            encoded_data, 'message=test'
+        self.gcm.make_request(
+            {'message': 'test'}, is_json=True
         )
-
-        # Assert return value
-        self.assertEquals(
-            response,
-            'blah'
+        self.assertEqual(self.gcm.headers['Content-Type'],
+            'application/json'
         )
+        self.assertTrue(mock_request.return_value.json.called)
 
 
-    @patch('urllib2.urlopen')
-    def test_make_request_unicode(self, urlopen_mock):
-        """ Regression: Test make_request with unicode payload. """
+    @patch('requests.post')
+    def test_make_request_plaintext(self, mock_request):
+        """ Test plaintext make_request. """
 
-        # Unicode character in data
+        mock_request.return_value.status_code = 200
+        mock_request.return_value.content = "OK"
+        # Perform request
+        response = self.gcm.make_request(
+            {'message': 'test'}, is_json=False
+        )
+        self.assertEqual(response, "OK")
+
+        mock_request.return_value.status_code = 400
+        with self.assertRaises(GCMMalformedJsonException):
+            response = self.gcm.make_request(
+                {'message': 'test'}, is_json=False
+            )
+
+        mock_request.return_value.status_code = 401
+        with self.assertRaises(GCMAuthenticationException):
+            response = self.gcm.make_request(
+                {'message': 'test'}, is_json=False
+            )
+
+        mock_request.return_value.status_code = 503
+        with self.assertRaises(GCMUnavailableException):
+            response = self.gcm.make_request(
+                {'message': 'test'}, is_json=False
+            )
+
+    @patch('requests.api.request')
+    def test_make_request_unicode(self, mock_request):
+        """ Test make_request with unicode payload. """
         data = {
             'message': u'\x80abc'
         }
-
-        self.gcm.make_request(data, is_json=False)
+        try:
+            self.gcm.make_request(data, is_json=False)
+        except:
+            pass
+        self.assertTrue(mock_request.called)
+        self.assertEqual(
+            mock_request.call_args[1]['data'],
+            'message=%C2%80abc'
+        )
 
     def test_retry_plaintext_request_ok(self):
         returns = [GCMUnavailableException(), GCMUnavailableException(), 'id=123456789']
